package com.weltevree.salvo.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

import com.weltevree.salvo.core.internal.Article;
import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.model.INewsgroup;

public class ArticleFactory {
	
	/**
	 * Factory method to get the article from a XOVER response. The overview
	 * header must have already been set in the newsgroup.
	 * 
	 * @see INewsgroup#getOverviewHeaders()
	 * @param responseValue
	 * @param newsgroup
	 * @return the new article or null if the article could not be created
	 */
	public static IArticle createArticle(String xoverResponseValue,
			INewsgroup newsgroup) {

		String[] values = xoverResponseValue.split("\t");
		String[] headers = newsgroup.getServer().getOverviewHeaders().toArray(
				new String[0]);
		if ((values.length - 1) != headers.length) {
			Debug.log(Article.class,
					"Invalid match between overview header and XOVER response");
			return null;
		}
		IArticle article = new Article(Integer.valueOf(values[0]), newsgroup,
				true);
		for (int ix = 1; ix < values.length; ix++) {
			article.setHeaderAttributeValue(headers[ix - 1], values[ix]);
		}

		return article;

	}
	

	public static IArticle createArticle(File file) {
		if (file.exists()) {
			try {
				FileInputStream f_in = new FileInputStream(file);
				ObjectInputStream obj_in = new ObjectInputStream(f_in);
				Object obj = obj_in.readObject();
				if (obj instanceof IArticle) {
					return (IArticle) obj;
				}
			} catch (Exception e) {
			}
		}
		return null;
	}

}
