package com.weltevree.salvo.core.internal;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.model.INewsgroup;
import com.weltevree.salvo.model.IServer;

public class Article implements IArticle {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private int articleNumber;

	private HashMap<String, String> headerAttributes = new HashMap<String, String>();

	private IArticle parent;

	private transient INewsgroup newsgroup;

	private boolean read;

	private boolean posted;

	private boolean isReplyRead = true;

	private HashMap<String, String> properties;

	private boolean isCommenting;

	private boolean marked;

	public Article(int articleNumber, INewsgroup newsgroup, boolean posted) {
		this.articleNumber = articleNumber;
		this.newsgroup = newsgroup;
		this.posted = posted;
	}

	public int getArticleNumber() {
		return articleNumber;
	}

	public String getDate() {
		return headerAttributes.get("Date:");
	}

	public String getFrom() {
		return headerAttributes.get("From:");
	}

	public String getHeaderAttributeValue(String attribute) {
		return headerAttributes.get(attribute);
	}

	public String getMessageId() {
		return headerAttributes.get("Message-ID:");
	}

	public INewsgroup getNewsgroup() {
		return newsgroup;
	}

	public IArticle getParent() {
		return parent;
	}

	public String[] getReferences() {
		String[] result = getHeaderAttributeValue("References:").trim().split(
				"\\s+?");
		if (result[0].length() == 0)
			return new String[0];
		return result;
	}

	public IServer getServer() {
		return newsgroup.getServer();
	}

	public String getSubject() {
		return getHeaderAttributeValue("Subject:");
	}

	public String getXRef() {
		return getHeaderAttributeValue("Xref:");
	}

	public void setHeaderAttributeValue(String element, String value) {
		String[] splitis = element.split(":");
		String[] valuesSplit = value.split(splitis[0] + ":");

		if (splitis.length > 1 && splitis[1].equals("full")) {
			headerAttributes.put(splitis[0] + ":", valuesSplit[1]);
		} else {
			headerAttributes.put(element, value.trim());
		}

	}

	public String getFormattedSize() {
		return (getSize() / 1024) + "KB";
	}

	public int getSize() {
		return Integer.valueOf(headerAttributes.get("Bytes:").trim());
	}

	public String getFullUserName() {

		// 
		// Simplistic implementation of rfc850
		//

		String[] split = getFrom().split("\\(");
		if (split.length > 1)
			return split[1].replace(')', ' ').trim();

		split = getFrom().split("\\<");
		if (split.length > 1)
			return split[0].trim();

		return getFrom();

	}

	public boolean isRead() {
		return read;
	}

	public void setRead(boolean read) {
		this.read = read;
	}

	public boolean isPosted() {
		return posted;
	}

	public void setNewsgroup(INewsgroup newsgroup) {
		this.newsgroup = newsgroup;
	}

	public void setParent(IArticle article) {
		parent = article;
	}

	public String getLastReference() {
		int size = getReferences().length;
		if (size == 0)
			return null;
		return getReferences()[size - 1];
	}

	public String[] getHeaderAttributes() {
		return headerAttributes.keySet().toArray(new String[0]);
	}

	public boolean isMine() {
		return getFullUserName().equals(getServer().getUser());
	}

	public boolean isReplyRead() {
		return isReplyRead;
	}

	public void setReplyRead(boolean read) {
		isReplyRead = read;
	}

	public Map<String, String> getProperties() {
		if (properties == null) {
			properties = new HashMap<String, String>();
		}
		return properties;
	}

	public String getProperty(String key) {
		return getProperties().get(key);
	}

	public void setProperty(String key, String value) {
		if (value == null) {
			getProperties().remove(key);
			return;
		}
		getProperties().put(key, value);
	}

	public void setThreadAttributes(Collection<IArticle> thread) {
		setReplyRead(true);
		for (IArticle reply : thread) {
			if (!reply.isRead()) {
				setReplyRead(false);
				break;
			}
		}

		setCommenting(false);
		for (IArticle reply : thread) {
			if (reply.isMine()) {
				setCommenting(true);
				break;
			}
		}
	}

	public void setCommenting(boolean isCommenting) {
		this.isCommenting = isCommenting;
	}

	public boolean isCommenting() {
		return isCommenting;
	}

	public boolean isMarked() {
		return this.marked;
	}

	public void setMarked(boolean marked) {
		this.marked = marked;
	}

	public boolean isReply() {
		return getReferences().length > 0;
	}
}
