package com.weltevree.salvo.core.internal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.model.INewsgroup;
import com.weltevree.salvo.model.IServer;
import com.weltevree.salvo.model.IServerConnection;
import com.weltevree.salvo.model.SALVO;

public class Server implements IServer {

	private static HashMap<String, IServer> servers;

	private final String address;

	private final int port;

	private final String user;

	private final boolean secure;

	private ServerConnection connection;

	private int batchSize;

	private transient Collection<String> overviewHeaders;

	private String email;

	private final String logIn;

	private Server(String address, int port, String user, String email, String logIn, String passWord,
			boolean secure) {
		this.address = address;
		this.port = port;
		this.user = user;
		this.email = email;
		this.logIn = logIn;

		this.secure = secure;

		connection = new ServerConnection(this);
		connection.connect(passWord);

	}

	public int getPort() {
		return port;
	}

	public String getAddress() {
		return address;
	}

	public String getLogin() {
		return logIn;
	}

	/**
	 * Factory method for creating the server. If the server is created it must
	 * be stored and reused.
	 * 
	 * @param address
	 * @param port
	 * @param user
	 * @param passWord
	 * @return an instance of {@link IServer} or null
	 */
	public static IServer getServer(String address, int port, String user, String email, String login,
			String passWord, boolean secure) {

		if (servers == null)
			servers = new HashMap<String, IServer>();

		IServer server = servers.get(address + "::" + port + "::" + user + "::" + email + "::" + login + "::"
				+ secure);

		if (server == null)
			server = new Server(address, port, user, email, login, passWord, secure);

		servers.put(server.toString(), server);
		return server;
	}

	@Override
	public String toString() {
		return getAddress() + "::" + getPort() + "::" + getUser() + "::" + getEmail() + "::" + getLogin()
				+ "::" + isSecure();
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof IServer)
			return toString().equals(obj.toString());
		return super.equals(obj);
	}

	@Override
	public int hashCode() {
		return toString().hashCode();
	}

	public boolean isSecure() {
		return secure;
	}

	public IServerConnection getServerConnection() {
		return connection;
	}

	public boolean isAnonymous() {
		return getLogin() == null;
	}

	public Map<String, IArticle> getBatch(int batchNumber) {
		// TODO Auto-generated method stub
		return null;
	}

	public int getBatchsize() {
		// TODO Auto-generated method stub
		if (batchSize == 0)
			return SALVO.DEFAULT_BATCH;
		return batchSize;
	}

	public int getNextBatch(INewsgroup group) {
		// TODO Auto-generated method stub
		return 0;
	}

	public void setBatchSize(int size) {
		batchSize = size;

	}

	public Collection<String> getOverviewHeaders() {
		if (overviewHeaders == null)
			overviewHeaders = new ArrayList<String>();
		return overviewHeaders;
	}

	public String getEmail() {
		return email;
	}

	public String getUser() {
		return user;
	}
}
