package com.weltevree.salvo.core.internal;

import java.util.ArrayList;
import java.util.Collection;

import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.model.INewsgroup;
import com.weltevree.salvo.model.IServer;
import com.weltevree.salvo.model.IServerStoreFacade;
import com.weltevree.salvo.model.IStore;
import com.weltevree.salvo.model.SALVO;

public class ServerStoreFacade implements IServerStoreFacade {

	private UpdateRunner updateRunner;

	private Collection<IStore> stores;

	public ServerStoreFacade(Collection<IStore> collection) {
		this.stores = collection;
	}

	public void init() {
		startUpdateThread();
	}

	private void startUpdateThread() {

		if (updateRunner == null) {
			updateRunner = new UpdateRunner();
		}

		if (!updateRunner.isThreadRunning())
			new Thread(updateRunner, "Salvo newsreader update thread").start();

	}

	public Collection<IStore> getStores() {
		return stores;
	}

	public boolean postArticle(IArticle article) {
		return false;
	}

	public IArticle fetchArticle(INewsgroup newsgroup, int articleId,
			int fetchType) {

		IArticle article = null;
		for (IStore store : stores) {
			store.fetchArticle(newsgroup, articleId, fetchType);
			break;
		}

		if (article == null) {
			return ServerRegex.fetchArticle(newsgroup, articleId, fetchType);
		}
		return article;
	}

	public Exception getLastException() {
		return null;
	}

	public void catchUp(INewsgroup newsgroup) {
		ServerRegex.getArticles(newsgroup, newsgroup.getLowWaterMark(),
				newsgroup.getHighWaterMark());

	}

	public boolean unsubscribeNewsgroup(INewsgroup newsGroup, boolean permanent) {
		for (IStore store : stores) {
			store.unsubscribeNewsgroup(newsGroup, permanent);
		}
		return true;

	}

	public boolean cancelArticle(IArticle article) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean subscribeNewsgroup(INewsgroup group) {
		for (IStore store : stores) {
			store.subscribeNewsgroup(group);
		}
		updateAttributes(group);
		return true;
	}

	public boolean subscribeServer(IServer server, String passWord) {
		for (IStore store : stores) {
			store.subscribeServer(server, passWord);
		}
		return true;
	}

	public boolean unsubscribeServer(IServer server, boolean permanent) {
		for (IStore store : stores) {
			store.unsubscribeServer(server, permanent);
		}
		return true;
	}

	public boolean updateAttributes(INewsgroup newsgroup) {
		ServerRegex.setWaterMarks(newsgroup);
		for (IStore store : stores) {
			store.updateAttributes(newsgroup);
		}
		return true;
	}

	public Collection<INewsgroup> getSubscribedNewsgroups(IServer server) {
		for (IStore store : stores) {
			return store.getSubscribedNewsgroups(server);
		}
		return new ArrayList<INewsgroup>();
	}

	public Collection<IArticle> getArticles(INewsgroup newsgroup, int from,
			int to) {

		// Adjust for sanity
		if ((to - from) > SALVO.BATCH_SIZE)
			from = to - SALVO.BATCH_SIZE;

		// Check what is first in store
		IStore firstStore = getFirstStore();
		int firstInStore = 0;
		if (firstStore != null && firstStore.getFirstArticle(newsgroup) != null)
			firstInStore = getFirstStore().getFirstArticle(newsgroup)
					.getArticleNumber();

		// Fetch from the server what is not in store
		if (firstInStore > 0 && firstInStore > from) {
			Collection<IArticle> result = ServerRegex.getArticles(newsgroup,
					from, firstInStore);
			if (result != null) {
				for (IStore store : stores)
					store.storeArticles(result);
			}

			// Adjust the requested values
			if (firstStore != null)
				firstInStore = firstStore.getFirstArticle(newsgroup)
						.getArticleNumber();
			if (firstInStore > from)
				from = firstInStore;
		}

		// Check what is last in store
		int lastInStore = 0;
		if (firstStore != null && firstStore.getLastArticle(newsgroup) != null)
			lastInStore = firstStore.getLastArticle(newsgroup)
					.getArticleNumber();

		// Fetch from the server what is not in store
		if (lastInStore > 0 && lastInStore < to) {
			Collection<IArticle> result = ServerRegex.getArticles(newsgroup,
					lastInStore, to);
			if (result != null)
				for (IStore store : stores)
					store.storeArticles(result);

			// Adjust the requested values
			if (firstStore != null)
				lastInStore = firstStore.getLastArticle(newsgroup)
						.getArticleNumber();
			if (lastInStore < to)
				to = lastInStore;
		}

		Collection<IArticle> result = null;
		if (firstStore != null)
			result = firstStore.getArticles(newsgroup, from, to);

		if (result == null) {
			result = ServerRegex.getArticles(newsgroup, from, to);
			if (result != null)
				for (IStore store : stores) {
					store.storeArticles(result);
				}
			if (firstStore != null)
				result = firstStore.getArticles(newsgroup, from, to);
		}
		if (result == null)
			result = new ArrayList<IArticle>();
		return result;
	}

	public IStore getFirstStore() {
		if (!stores.isEmpty())
			return (IStore) stores.toArray()[0];
		return null;
	}

	public String[] getArticleBody(IArticle article) {

		// FIXME Decide if article bodies should be stored in the store or
		// always fetched from server.
		String[] body = null;
		if (getFirstStore() != null)
			body = getFirstStore().getArticleBody(article);
		if (body == null) {
			body = ServerRegex.getBody(article);
			if (!(body == null)) {
				for (IStore store : stores) {
					store.storeArticleBody(article, body);
				}
			}
		}
		return body;
	}

	public Collection<IArticle> getFollowUps(IArticle article) {

		// FIXME Decide if article bodies should be stored in the store or
		// always fetched from server.
		Collection<IArticle> result = null;
		if (getFirstStore() != null)
			result = getFirstStore().getFollowUps(article);
		if (result == null) {
			result = ServerRegex.getFollowUps(article);
			if (!(result == null)) {
				if (getFirstStore() != null)
					getFirstStore().storeArticles(result);
			}
		}
		return result;
	}

	public Collection<IArticle> getAllFollowUps(IArticle article) {

		// FIXME Decide if article bodies should be stored in the store or
		// always fetched from server.
		Collection<IArticle> result2 = getFollowUps(article);
		Collection<IArticle> result = new ArrayList<IArticle>(result2);
		for (IArticle reply : result) {
			Collection<IArticle> r2 = getAllFollowUps(reply);
			result2.addAll(r2);
		}
		return result2;
	}

	public void updateArticle(IArticle article) {
		for (IStore store : stores)
			store.updateArticle(article);
	}

	public boolean replyToArticle(IArticle article, StringBuffer buffer) {

		if (ServerRegex.replyToArticle(article, buffer)) {
			updateAttributes(article.getNewsgroup());
			return true;
		}
		return false;
	}

	public boolean postNewArticle(INewsgroup[] newsgroups, String subject,
			StringBuffer buffer) {
		if (ServerRegex.postNewArticle(newsgroups, subject, buffer)) {
			for (INewsgroup newsgroup : newsgroups) {
				updateAttributes(newsgroup);
			}
			return true;
		}
		return false;
	}
}
