package com.weltevree.salvo.model;

import java.util.Collection;
import java.util.Map;

public interface IServer {

	/**
	 * Returns the port.
	 * 
	 * @return int
	 */
	public int getPort();

	/**
	 * Returns the TCP/IP address.
	 * 
	 * @return String
	 */
	public String getAddress();

	/**
	 * Returns the login.
	 * 
	 * @return String containing the login or null of this is an anonymous
	 *         connection.
	 */
	public String getLogin();
	
	/**
	 * Returns the user.
	 * 
	 * @return String containing the user name.
	 */
	public String getUser();

	/**
	 * Returns the {@link IServerConnection} object used for communications.
	 * 
	 * @return the {@link IServerConnection} object used for communcations.
	 *         Cannot be null.
	 */
	public IServerConnection getServerConnection();

	/**
	 * Indicates that this server communicates over a secure layer (SSL/TSL).
	 * 
	 * @return
	 */
	public boolean isSecure();

	/**
	 * Returns true if this is an anonymous server connection.
	 * 
	 * @return boolean
	 */
	public boolean isAnonymous();

	/**
	 * Sets the batch size of article headers to retrieve from the news server.
	 * If not set then it defaults to SALVO.DEFAULT_BATCH
	 * 
	 * @param size
	 *            the batch size
	 */
	public void setBatchSize(int size);

	/**
	 * Returns the batch size.
	 * 
	 * @see IServer#setBatchSize(int)
	 * @return the batch size as int
	 */
	public int getBatchsize();

	/**
	 * Loads the next batch of articles from the server and returns the batch
	 * number. The batch can be fetched by #getBatch()`
	 * 
	 * @return the batch number as int
	 */
	public int getNextBatch(INewsgroup group);

	/**
	 * Gets the batch of articles with the specified batch number.
	 * 
	 * @param batchNumber
	 *            is the int that defines the batchnumber to fetch.
	 * @return the batch or null if the batch does not exist
	 */
	public Map<String, IArticle> getBatch(int batchNumber);
	
	/**
	 * Returns a Collection of headers that are returned when the
	 * <code>LIST OVERVIEW.FMT</code> is issued. The internal Collection is
	 * passed so that clients can clear it.
	 * 
	 * @return Collection - The overview headers. May not be null.
	 */
	public Collection<String> getOverviewHeaders();

	public String getEmail();
}
