package com.weltevree.salvo.model;

import java.util.Collection;

public interface IServerConnection {

	/**
	 * Connects to the server. If the server could not connect you could try
	 * getting the last exception for more information.
	 * 
	 * @see #getLastException()
	 * 
	 * @param password
	 * @return true if the server was connected, false otherwise
	 */
	public boolean connect(String password);

	/**
	 * Disconnects from the server. If the server could not disconnect you could
	 * try getting the last exception for more information.
	 * 
	 * @see #getLastException()
	 * 
	 * @return true if the server was disconnected, false otherwise
	 */
	public boolean disconnect();

	public IServer getServer();

	public Collection<INewsgroup> getNewsgroups();

	public boolean isConnected();

	/**
	 * Sends a command to the server returns true if the command was send in
	 * which case the response can be had from {@link #getresponse()}
	 * 
	 * @param command
	 * @return true if all clear or false in case of problems
	 */
	public boolean sendCommand(String command);

	/**
	 * Retrieves the response send by the last {@link #sendCommand(String)}.
	 * 
	 * @return String response
	 */
	public String getResponse();

	/**
	 * Retrieves the last response from the server.
	 * 
	 * @return String the last known response
	 */
	public String getLastResponse();

	/**
	 * Retrieves the response send by the last {@link #sendCommand(String)} and
	 * returns the response in an array. The response of this command must be a
	 * list. This means that the response is terminated with a dot and the CRLF
	 * sequence.
	 * 
	 * @see <a href="http://tools.ietf.org/html/rfc3977#section-3.1.1">RFC
	 *      Section 3.1.1</a>
	 * 
	 * @return the response in a String[]
	 */
	public String[] getListResponse();


	/**
	 * Fetches the bodytext for this article and stores it in the article.
	 * 
	 * @param article
	 */
	public void getBody(IArticle article);

	Exception getLastException();

	public void flush();

}
