package com.weltevree.salvo.ui.internal.editor;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableColorProvider;
import org.eclipse.jface.viewers.ITableFontProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.ui.internal.resources.ISalvoResource;

public class ArticlePaneLabelProvider implements ITableLabelProvider,
		ITableFontProvider, ITableColorProvider {

	private static Font unreadArticle;
	private static Font unreadReplies;
	private static Color mineColor;
	private static Color commentColor;

	static {

		Font font = JFaceResources.getDefaultFont();

		FontData fd = font.getFontData()[0];
		fd.setStyle(SWT.BOLD);
		unreadArticle = new Font(font.getDevice(), fd);

		fd = font.getFontData()[0];
		fd.setStyle(SWT.ITALIC);
		unreadReplies = new Font(font.getDevice(), fd);

		mineColor = Display.getDefault().getSystemColor(SWT.COLOR_RED);
		commentColor = Display.getDefault().getSystemColor(SWT.COLOR_DARK_MAGENTA);
	}

	public Image getColumnImage(Object element, int columnIndex) {
		return null;
	}

	public String getColumnText(Object element, int columnIndex) {

		if (element instanceof ISalvoResource) {

			IArticle article = (IArticle) ((ISalvoResource) element)
					.getObject();

			if (columnIndex == 0)
				return article.getSubject();
			if (columnIndex == 1)
				return article.getFullUserName();
			if (columnIndex == 2)
				return article.getDate();
			if (columnIndex == 3)
				return article.getFormattedSize();
			if (columnIndex == 4)
				return "?6?";

		}

		return "dunno";
	}

	public void addListener(ILabelProviderListener listener) {
	}

	public void dispose() {
	}

	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	public void removeListener(ILabelProviderListener listener) {
	}

	public Font getFont(Object element, int columnIndex) {

		IArticle article = (IArticle) ((ISalvoResource) element).getObject();

		if (columnIndex == 0) {
			if (!article.isRead()) {
				return unreadArticle;
			}
			if (!article.isReplyRead()) {
				return unreadReplies;
			}
		}

		return JFaceResources.getDefaultFont();
	}

	public Color getBackground(Object element, int columnIndex) {
		return null;
	}

	public Color getForeground(Object element, int columnIndex) {
		IArticle article = (IArticle) ((ISalvoResource) element).getObject();

		if (columnIndex == 0) {
			if (article.isCommenting()) {
				return commentColor;
			}
			if (article.isMine()) {
				return mineColor;
			}
		}
		return null;
	}
}