package com.weltevree.salvo.ui.internal.editor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.layout.TreeColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.model.INewsgroup;
import com.weltevree.salvo.ui.internal.Activator;
import com.weltevree.salvo.ui.internal.resources.ISalvoResource;

/**
 * This editor fills a lazy table tree with the article information and to the
 * left of this table a static table is attached that contains information icons
 * like follow and attachments indicators.
 * 
 * @author wim.jongman@gmail.com
 * 
 */
public class ArticlePanel extends EditorPart implements ISelectionListener {

	/**
	 * Calculate the index match between the static side table and the dynamic
	 * main table and then annotate the article in the side table.
	 * 
	 * @author wim.jongman@gmail.com
	 * 
	 */
	private final class ArticlePanelSideTablePaintListener implements
			PaintListener {

		public void paintControl(PaintEvent e) {

			try {

				// Stop & start drawing of the main tree seems to trigger
				// flooding of paint events.
				// tree.setRedraw(false);

				// Stop the sidetable from flickering
				sideTable.setVisible(false);

				// Calculate the current visible top and bottom indices in the
				// main tree.
				int itemHeight = tree.getItemHeight();

				// Walk over the current visible articles and set/remove the
				// icons in the side tree
				int counter = 0;
				while (true) {

					TreeItem itemd = tree.getItem(new Point(0, counter++
							* itemHeight));
					if (itemd == null || itemd.getData() == null)
						return;

					// Make the side tree just as long as the (resized) main
					// tree
					sideTable.setItemCount(counter);

					IArticle article = (IArticle) ((ISalvoResource) itemd
							.getData()).getObject();

					// Set some icons
					if (!article.isReply() && article.isCommenting() || article.isMine())
						sideTable.getItem(counter-1).setImage(
								1,
								Activator.getDefault().getImageRegistry().get(
										"messages.gif"));
					else {
						sideTable.getItem(counter-1).setImage(1, null);
					}

					if (article.isMarked())
						sideTable.getItem(counter-1).setImage(
								2,
								Activator.getDefault().getImageRegistry().get(
										"follow.gif"));
					else {
						sideTable.getItem(counter-1).setImage(2, null);
					}
				}
			} finally {
				sideTable.pack();
				sideTable.setVisible(true);
				// tree.setVisible(true);
			}
		}
	}

	public ArticlePanel() {
	}

	private Tree sideTable;

	private TreeViewer treeViewer;

	public static final String ID = "com.weltevree.salvo.ui.articlepanel";

	private Tree tree;

	private INewsgroup savedNewsgroup;

	@Override
	public void doSave(IProgressMonitor monitor) {
	}

	@Override
	public void doSaveAs() {
	}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	@Override
	public boolean isDirty() {
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void createPartControl(Composite parent) {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		parent.setLayout(gridLayout);

		final ToolBar toolBar = new ToolBar(parent, SWT.NONE);

		final ToolItem newItemToolItem = new ToolItem(toolBar, SWT.PUSH);
		newItemToolItem.setText("New item");

		final ToolItem newItemToolItem_1 = new ToolItem(toolBar, SWT.PUSH);
		newItemToolItem_1.setText("New item");

		// composite_1.setSize(494, 174);

		TreeColumnLayout layout1 = new TreeColumnLayout();
		TreeColumnLayout layout2 = new TreeColumnLayout();

		final Composite tableComposite = new Composite(parent, SWT.NONE);
		tableComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true));
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		gridLayout_1.verticalSpacing = 0;
		gridLayout_1.marginWidth = 0;
		gridLayout_1.marginHeight = 0;
		gridLayout_1.horizontalSpacing = 0;
		tableComposite.setLayout(gridLayout_1);

		final Composite tree1Composite = new Composite(tableComposite, SWT.NONE);
		tree1Composite.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false,
				true));

		sideTable = new Tree(tree1Composite, SWT.NONE);
		sideTable.setLinesVisible(true);
		sideTable.setHeaderVisible(true);

		final TreeColumn column1 = new TreeColumn(sideTable, SWT.NONE);
		layout1.setColumnData(column1, new ColumnWeightData(00));

		final TreeColumn column2 = new TreeColumn(sideTable, SWT.NONE);
		column2.setWidth(18);
		layout1.setColumnData(column2, new ColumnWeightData(50));

		final TreeColumn column3 = new TreeColumn(sideTable, SWT.NONE);
		column3.setWidth(18);
		layout1.setColumnData(column3, new ColumnWeightData(50));

		tree1Composite.setLayout(layout1);
		column1.setWidth(0);

		final Composite tree2Composite = new Composite(tableComposite, SWT.NONE);
		tree2Composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true));
		final GridLayout gridLayout_3 = new GridLayout();
		gridLayout_3.marginWidth = 0;

		treeViewer = new TreeViewer(tree2Composite, SWT.VIRTUAL
				| SWT.FULL_SELECTION);
		tree = treeViewer.getTree();
		tree.setLinesVisible(true);
		treeViewer.setUseHashlookup(true);
		treeViewer.setLabelProvider(new ArticlePaneLabelProvider());
		treeViewer.setContentProvider(new ArticlePanelContentProvider());
		tree.setHeaderVisible(true);
		tree.addPaintListener(new ArticlePanelSideTablePaintListener());

		final TreeColumn newColumnTreeColumn_2 = new TreeColumn(tree, SWT.NONE);
		newColumnTreeColumn_2.setText("subject");
		layout2.setColumnData(newColumnTreeColumn_2, new ColumnWeightData(40));

		final TreeColumn newColumnTreeColumn_3 = new TreeColumn(tree, SWT.NONE);
		newColumnTreeColumn_3.setText("from");
		layout2.setColumnData(newColumnTreeColumn_3, new ColumnWeightData(25));
		newColumnTreeColumn_3.setMoveable(true);

		final TreeColumn newColumnTreeColumn_4 = new TreeColumn(tree, SWT.NONE);
		newColumnTreeColumn_4.setText("sent");
		layout2.setColumnData(newColumnTreeColumn_4, new ColumnWeightData(30));
		newColumnTreeColumn_4.setMoveable(true);

		final TreeColumn newColumnTreeColumn_5 = new TreeColumn(tree, SWT.NONE);
		newColumnTreeColumn_5.setText("size");
		layout2.setColumnData(newColumnTreeColumn_5, new ColumnWeightData(5));
		newColumnTreeColumn_5.setMoveable(true);

		tree2Composite.setLayout(layout2);

		getSite().setSelectionProvider(treeViewer);
		addContextMenu(treeViewer);
		getSite().getWorkbenchWindow().getSelectionService()
				.addPostSelectionListener(this);

		setInput(((ArticlePanelInput) getEditorInput()).getNewsgroup());

	}

	private void addContextMenu(TreeViewer viewer) {

		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager mgr) {
				fillContextMenu(mgr);
			}
		});

		// Create menu.
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// Register menu for extension.
		getSite().registerContextMenu(menuMgr, viewer);

	}

	protected void fillContextMenu(IMenuManager mgr) {
		mgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	@Override
	public void dispose() {
		getSite().getWorkbenchWindow().getSelectionService()
				.removePostSelectionListener(this);
		super.dispose();
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}

	private void setInput(INewsgroup group) {

		// TODO Spin of a Job that loads the articles in the newsgroup
		// new NewsgroupArticleLoaderJob(group).schedule();

		if (group != savedNewsgroup) {
			savedNewsgroup = group;
			((ArticlePanelInput) getEditorInput()).setNewsGroup(group);
			setPartName(group.getNewsgroupName());
			treeViewer.getTree().clearAll(true);
			treeViewer.setInput(group);
		}
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {

		if (part == this) {
			return;
		}

		if (selection instanceof IStructuredSelection) {
			if (((IStructuredSelection) selection).getFirstElement() instanceof ISalvoResource) {
				if (((ISalvoResource) ((IStructuredSelection) selection)
						.getFirstElement()).getObject() instanceof INewsgroup) {
					setInput((INewsgroup) ((ISalvoResource) ((IStructuredSelection) selection)
							.getFirstElement()).getObject());
				}
			}
		}
	}

	public void updateArticle(ISalvoResource resource) {
		treeViewer.update(resource, null);
	}
}
