package com.weltevree.salvo.ui.internal.views;

import java.io.ByteArrayInputStream;

import org.apache.james.mime4j.parser.MimeStreamParser;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.commands.ActionHandler;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISaveablePart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.ViewPart;

import com.weltevree.salvo.core.ServerStoreFactory;
import com.weltevree.salvo.model.IArticle;
import com.weltevree.salvo.model.IServerStoreFacade;
import com.weltevree.salvo.model.SALVO;
import com.weltevree.salvo.ui.internal.MimeArticleContentHandler;
import com.weltevree.salvo.ui.internal.resources.ISalvoResource;
import com.weltevree.salvo.ui.tools.SelectionUtil;

public class ReplyView extends ViewPart implements ISaveablePart {

	private Text bodyText;

	private boolean dirty = false;

	private IArticle article;

	public ReplyView() {

	}

	@Override
	public void dispose() {
	}

	@Override
	public void createPartControl(Composite parent) {

		bodyText = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.WRAP);

		ISalvoResource resource = (ISalvoResource) SelectionUtil
				.getFirstObjectFromCurrentSelection(ISalvoResource.class);

		if (resource != null && resource.getObject() instanceof IArticle) {

			// FIXME same code is used in MessageView

			article = (IArticle) resource.getObject();

			StringBuffer buffer = new StringBuffer();
			String[] body = (String[]) ServerStoreFactory.instance().getServerStoreFacade().getArticleBody(
					article);
			for (String line : body) {
				buffer.append(line + SALVO.CRLF);
			}

			MimeArticleContentHandler handler = new MimeArticleContentHandler(article);
			MimeStreamParser parser = new MimeStreamParser();
			parser.setContentHandler(handler);

			try {
				parser.parse(new ByteArrayInputStream(buffer.toString().getBytes()));
			} catch (Exception ex) {
				ex.printStackTrace();
			}

			// for (Control child : parent.getChildren()) {
			// child.dispose();
			// }

			buffer = new StringBuffer();
			buffer.append(" " + SALVO.CRLF);
			buffer.append(" " + SALVO.CRLF);
			buffer.append(" " + SALVO.CRLF);
			for (String line : handler.getBody().split(SALVO.CRLF)) {
				if (line.startsWith(">"))
					buffer.append(">" + line + SALVO.CRLF);
				else
					buffer.append("> " + line + SALVO.CRLF);
			}

			bodyText.setText(buffer.toString());
			final int textChangeHash = bodyText.getText().hashCode();
			bodyText.addKeyListener(new KeyAdapter() {

				@Override
				public void keyReleased(KeyEvent e) {
					dirty = false;
					System.out.println(textChangeHash + " - " + bodyText.getText().hashCode());
					if (textChangeHash != bodyText.getText().hashCode()) {
						dirty = true;
					}
					firePropertyChange(PROP_DIRTY);
				}
			});

			setContentDescription("Reply to: " + article.getNewsgroup());
			setPartName(article.getSubject().startsWith("Re: ") ? article.getSubject() : "Re: "
					+ article.getSubject());
		}

		IHandlerService handlerService = (IHandlerService) getViewSite().getService(IHandlerService.class);
		handlerService.activateHandler("org.eclipse.ui.file.save", new ActionHandler(ActionFactory.SAVE
				.create(getSite().getWorkbenchWindow())));

		// getViewSite().getActionBars().setGlobalActionHandler("org.eclipse.ui.file.save",
		// ActionFactory.SAVE.create(getSite().getWorkbenchWindow()));

	}

	@Override
	public void setFocus() {
		bodyText.setFocus();
		bodyText.forceFocus();
	}

	public void doSave(IProgressMonitor monitor) {

		final StringBuffer buffer = new StringBuffer(bodyText.getText());

		monitor.beginTask("Posting", 43);

		for (int i = 0; i < 5; i++) {
			monitor.subTask("You have " + (5 - i) + " seconds to cancel");
			for (int i2 = 0; i2 < 10; i2++) {
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					monitor.setCanceled(true);
				}
				if (monitor.isCanceled()) {
					monitor.setCanceled(true);
					return;
				}
				monitor.worked(1);
			}
		}

		monitor.subTask("Posting to newsgroup " + article.getNewsgroup().getNewsgroupName());
		monitor.worked(1);
		IServerStoreFacade serverStoreFacade = ServerStoreFactory.instance().getServerStoreFacade();
		monitor.worked(1);

		boolean success = serverStoreFacade.replyToArticle(article, buffer);
		monitor.done();
		if (success) {
			dirty = false;
			firePropertyChange(PROP_DIRTY);
			final ReplyView view = this;
			getViewSite().getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					getViewSite().getPage().hideView(view);
				}
			});
		}

		else {
			MessageDialog.openError(getViewSite().getShell(), "Problem posting message",
					"The message could not be posted.");
		}
	}

	public void doSaveAs() {
	}

	public boolean isDirty() {
		return dirty;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public boolean isSaveOnCloseNeeded() {
		return true;
	}
}
