package com.weltevree.salvo.ui.internal.wizards;

import java.net.UnknownHostException;

import org.eclipse.equinox.security.storage.ISecurePreferences;
import org.eclipse.equinox.security.storage.SecurePreferencesFactory;
import org.eclipse.equinox.security.storage.StorageException;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.weltevree.salvo.core.Debug;
import com.weltevree.salvo.core.ServerFactory;
import com.weltevree.salvo.model.IServer;
import com.weltevree.salvo.model.IServerConnection;

public class NewNewsServerWizardPage extends WizardPage {

	private Text pass;

	private Text user;

	private Text login;

	private Text email;

	private Text address;

	private Text port;

	// private boolean isPageComplete = false;
	private Button requiresLogOnButton;

	private Button validateButton;

	private boolean serverValidated;

	private Label logInLabel;

	private Label passwordLabel;

	protected NewNewsServerWizardPage(String pageName) {
		super(pageName);
		setTitle("New Server");
		setDescription("Enter the news server information and press Validate.");
	}

	public void createControl(Composite parent) {

		ModifyListener editor = new NewNewsServerWizardPageEditor(this);

		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		composite.setLayout(new GridLayout(2, false));

		Label label = new Label(composite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		label.setText("News Server Address");

		address = new Text(composite, SWT.SINGLE | SWT.LEAD | SWT.BORDER);
		address.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		address.setText("news.eclipse.org");

		Label label2 = new Label(composite, SWT.NONE);
		label2.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		label2.setText("Port");

		port = new Text(composite, SWT.SINGLE | SWT.LEAD | SWT.BORDER);
		port.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		port.setText("119");

		Label label21 = new Label(composite, SWT.NONE);
		label21.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		label21.setText("User Name");

		user = new Text(composite, SWT.SINGLE | SWT.LEAD | SWT.BORDER);
		user.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		user.setText("First Last");

		Label label22 = new Label(composite, SWT.NONE);
		label22.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		label22.setText("Email Address");

		email = new Text(composite, SWT.SINGLE | SWT.LEAD | SWT.BORDER);
		email.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		email.setText("your email");

		setControl(composite);

		requiresLogOnButton = new Button(composite, SWT.CHECK);
		requiresLogOnButton.setSelection(false);
		new Label(composite, SWT.NONE);

		final Composite composite_1 = new Composite(composite, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite_1.setLayout(gridLayout);

		logInLabel = new Label(composite_1, SWT.NONE);
		logInLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		logInLabel.setText("Login");

		login = new Text(composite_1, SWT.BORDER);
		login.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		login.setText("exquisitus");
		login.setSelection(new Point(0, 9));
		login.setEnabled(false);

		passwordLabel = new Label(composite_1, SWT.NONE);
		passwordLabel.setText("Password");

		pass = new Text(composite_1, SWT.BORDER);
		pass.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		pass.setEchoChar('*');
		pass.setText("flinder1f7");
		pass.setEnabled(false);

		requiresLogOnButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				setLoginEnabled(requiresLogOnButton.getSelection());
			}
		});
		requiresLogOnButton.setText("Requires log on");

		new Label(composite, SWT.NONE);
		new Label(composite, SWT.NONE);

		validateButton = new Button(composite, SWT.NONE);
		validateButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(final MouseEvent e) {

				validateServer();

			}
		});
		validateButton.setLayoutData(new GridData());
		validateButton.setSelection(true);
		validateButton.setText("Validate");
		new Label(composite, SWT.NONE);

		fillDialog();

		address.addModifyListener(editor);
		port.addModifyListener(editor);
		user.addModifyListener(editor);
		email.addModifyListener(editor);
		login.addModifyListener(editor);
		pass.addModifyListener(editor);
	}

	protected void setLoginEnabled(boolean selection) {
		logInLabel.setEnabled(selection);
		login.setEnabled(selection);
		passwordLabel.setEnabled(selection);
		pass.setEnabled(selection);

	}

	private void fillDialog() {
		IServer server = ((NewNewsServerWizard) getWizard()).getServer();
		if (server == null) {
			return;
		}
		address.setText(server.getAddress());
		port.setText(server.getPort() + "");
		user.setText(server.getUser());
		email.setText(server.getEmail());
		requiresLogOnButton.setSelection(!server.isAnonymous());
		setLoginEnabled(!server.isAnonymous());
		if (!server.isAnonymous()) {
			login.setText(server.getLogin());
			ISecurePreferences prefs = SecurePreferencesFactory.getDefault();
			ISecurePreferences node = prefs.node("/com/weltevree/salvo");
			try {
				pass.setText(node.get(address.getText(), ""));
			} catch (StorageException e) {
				Debug.log(this.getClass(), "Storage Exception: " + e.getMessage());
				pass.setText("");
			}
		}
	}

	protected void validateServer() {
		// If everything is ok then try to connect to the server if this
		// is
		// required.
		//
		if (getErrorMessage() == null) {

			BusyIndicator.showWhile(null, new Runnable() {

				public void run() {
					setErrorMessage(null);

					IServer server = ServerFactory.getServer(getAddress(), getPort(), getUser(), getEmail(),
							getLogin(), getPass(), isSecure());
					IServerConnection connection = server.getServerConnection();
					connection.disconnect();
					connection.connect(getPass());

					if (!connection.isConnected()) {
						if (connection.getLastException().getClass() == UnknownHostException.class) {
							setErrorMessage("Unknown Host: " + connection.getLastException().getMessage());

						} else {
							setErrorMessage(connection.getLastException().getMessage());
						}
					} else if (connection.getLastResponse() != null
							&& !connection.getLastResponse().matches("(?s)[12].*")) {
						setErrorMessage(connection.getLastResponse()
								+ " (This could be cryptic but try to make something of it)");

					}
					setServerValidated(true);

					return;

				}
			});

		}

	}

	public String getEmail() {
		return email.getText();
	}

	public void setServerValidated(boolean b) {
		serverValidated = b;
		getWizard().getContainer().updateButtons();
	}

	@Override
	public boolean isPageComplete() {
		return serverValidated;
	}

	public String getAddress() {
		return address.getText();
	}

	public int getPort() {
		return Integer.valueOf(port.getText());
	}

	public String getUser() {
		return user.getText();
	}

	public String getLogin() {
		if (requiresLogOnButton.getSelection()) {
			return login.getText();
		}
		return null;
	}

	public boolean isLogonRequired() {
		return requiresLogOnButton.getSelection();
	}

	public boolean isValidateRequired() {
		return validateButton.getSelection();
	}

	public String getPass() {
		if (requiresLogOnButton.getSelection()) {
			return pass.getText();
		}
		return null;
	}

	public IServer getServer() {
		return ServerFactory.getServer(getAddress(), getPort(), getUser(), getEmail(), getLogin(), getPass(),
				isSecure());
	}

	public boolean isSecure() {
		// TODO Auto-generated method stub
		return false;
	}
}
