package com.weltevree.salvo.ui.internal.wizards;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

import com.weltevree.salvo.model.INewsgroup;
import com.weltevree.salvo.model.IServer;
import com.weltevree.salvo.ui.internal.provider.NewsContentProvider;
import com.weltevree.salvo.ui.internal.provider.NewsGroupProvider;
import com.weltevree.salvo.ui.internal.provider.NewsLabelProvider;
import com.weltevree.salvo.ui.internal.resources.ISalvoResource;
import com.weltevree.salvo.ui.internal.resources.SalvoResourceFactory;

public class SubscribeGroupWizardPage extends WizardPage {

	private Tree tree;
	private CheckboxTreeViewer checkboxTreeViewer;

	protected SubscribeGroupWizardPage(String pageName) {
		super(pageName);
		setDescription("Select the groups you want to subscribe to.");
	}

	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		composite.setLayout(new GridLayout(1, false));

		setControl(composite);

		checkboxTreeViewer = new CheckboxTreeViewer(composite, SWT.BORDER);
		tree = checkboxTreeViewer.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		checkboxTreeViewer.setContentProvider(new NewsContentProvider());
		checkboxTreeViewer.setLabelProvider(new NewsLabelProvider());
		checkboxTreeViewer.setComparator(new ViewerComparator());

	}

	public Collection<INewsgroup> getGroups() {

		Collection<INewsgroup> result = new ArrayList<INewsgroup>();
		if (checkboxTreeViewer == null)
			return result;

		for (Object object : checkboxTreeViewer.getCheckedElements()) {

			if (object instanceof ISalvoResource) {
				if (((ISalvoResource) object).getObject() instanceof INewsgroup) {
					result.add((INewsgroup) ((ISalvoResource) object).getObject());
				}
			}
		}
		return result;
	}

	public void setInput(IServer server) {
		setTitle(server.getAddress());
		ISalvoResource s2 = SalvoResourceFactory.getResource(server.getAddress(), server);
		s2.setChildProvider(new NewsGroupProvider(s2));
		checkboxTreeViewer.setInput(s2);
	}
}
